(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/common'), require('@angular/core')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/common', '@angular/core'], factory) :
	(factory((global.ngxMgauge = {}),global.ng.common,global.ng.core));
}(this, (function (exports,_angular_common,_angular_core) { 'use strict';

var DEFAULTS = {
    MIN: 0,
    MAX: 100,
    TYPE: 'arch',
    THICK: 10,
    FOREGROUND_COLOR: '#1e88e5',
    BACKGROUND_COLOR: '#e4e4e4',
    FILL_COLOR: '#bbdefb',
    CAP: 'round',
    SIZE: 150,
    DURATION: 2500,
    SHOW_EXTREMUM: true
};
var NgxMGaugeOptions = (function () {
    function NgxMGaugeOptions() {
        this.type = DEFAULTS.TYPE;
        this.cap = DEFAULTS.CAP;
        this.size = DEFAULTS.SIZE;
        this.thick = DEFAULTS.THICK;
        this.foregroundColor = DEFAULTS.FOREGROUND_COLOR;
        this.backgroundColor = DEFAULTS.BACKGROUND_COLOR;
        this.fillColor = DEFAULTS.FILL_COLOR;
        this.duration = DEFAULTS.DURATION;
        this.showExtremum = DEFAULTS.SHOW_EXTREMUM;
    }
    return NgxMGaugeOptions;
}());

function clamp(value, min, max) {
    return Math.max(min, Math.min(max, value));
}

function coerceNumberProperty(value, fallbackValue) {
    if (fallbackValue === void 0) { fallbackValue = 0; }
    return isNaN(parseFloat(value)) || isNaN(Number(value)) ? fallbackValue : Number(value);
}
function cssUnit(value) {
    return value + "px";
}
function isNumber(value) {
    return value !== undefined && !isNaN(parseFloat(value)) && !isNaN(Number(value));
}
var NgxMGaugeComponent = (function () {
    function NgxMGaugeComponent(_elementRef, _renderer) {
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        this.role = 'mgauge';
        this.gaugeMeterClass = true;
        this.ariaValueminAttr = 'min';
        this.ariaValuemaxAttr = 'max';
        this.ariaValueNowAttr = 'value';
        this._initialized = false;
        this._min = DEFAULTS.MIN;
        this._max = DEFAULTS.MAX;
        this._value = 0;
        this.duration = DEFAULTS.DURATION;
        this.showExtremum = DEFAULTS.SHOW_EXTREMUM;
    }
    Object.defineProperty(NgxMGaugeComponent.prototype, "min", {
        get: function () { return this._min; },
        set: function (value) {
            this._min = coerceNumberProperty(value, DEFAULTS.MIN);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgxMGaugeComponent.prototype, "max", {
        get: function () { return this._max; },
        set: function (value) {
            this._max = coerceNumberProperty(value, DEFAULTS.MAX);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgxMGaugeComponent.prototype, "value", {
        get: function () { return this._value; },
        set: function (val) {
            this._value = coerceNumberProperty(val);
        },
        enumerable: true,
        configurable: true
    });
    NgxMGaugeComponent.prototype.ngOnChanges = function (changes) {
        var isTextChanged = changes['label'] || changes['append'] || changes['prepend'];
        var isDataChanged = changes['value'] || changes['min'] || changes['max'];
        if (this._initialized) {
            if (isDataChanged) {
                this._refresh();
            }
            else if (!isTextChanged) {
                this._destroy();
                this._init();
            }
        }
    };
    NgxMGaugeComponent.prototype._refreshSize = function () {
        this._renderer.setElementStyle(this._elementRef.nativeElement, 'width', cssUnit(this.options.size));
        this._renderer.setElementStyle(this._elementRef.nativeElement, 'height', cssUnit(this.options.size));
    };
    NgxMGaugeComponent.prototype.ngAfterViewInit = function () {
        if (this._canvas) {
            this._init();
        }
    };
    NgxMGaugeComponent.prototype.ngOnDestroy = function () {
        this._destroy();
    };
    NgxMGaugeComponent.prototype._getBounds = function (type) {
        var head, tail;
        switch (type) {
            case 'semi':
                head = Math.PI;
                tail = 2 * Math.PI;
                break;
            case 'full':
                head = 1.5 * Math.PI;
                tail = 3.5 * Math.PI;
                break;
            case 'arch':
                head = 0.8 * Math.PI;
                tail = 2.2 * Math.PI;
                break;
        }
        return { head: head, tail: tail };
    };
    NgxMGaugeComponent.prototype._drawShell = function (start, middle, tail, color) {
        var center = this._getCenter(), radius = this._getRadius();
        this._clear();
        if (this._context2d) {
            this._context2d.beginPath();
            this._context2d.strokeStyle = this.options.backgroundColor;
            this._context2d.arc(center.x, center.y, radius, middle, tail, false);
            this._context2d.stroke();
            this._context2d.beginPath();
            this._context2d.strokeStyle = color;
            this._context2d.arc(center.x, center.y, radius, start, middle, false);
            this._context2d.stroke();
            this._context2d.beginPath();
            this._context2d.arc(center.x, center.y, radius - this.options.thick, 0, 2 * Math.PI, false);
            this._context2d.fillStyle = this.options.fillColor;
            this._context2d.fill();
        }
    };
    NgxMGaugeComponent.prototype._clear = function () {
        if (this._context2d) {
            this._context2d.clearRect(0, 0, this._getWidth(), this._getHeight());
        }
    };
    NgxMGaugeComponent.prototype._getWidth = function () {
        return this.options.size;
    };
    NgxMGaugeComponent.prototype._getHeight = function () {
        return this.options.size;
    };
    NgxMGaugeComponent.prototype._getRadius = function () {
        var center = this._getCenter();
        return center.x - this.options.thick;
    };
    NgxMGaugeComponent.prototype._getCenter = function () {
        var x = this._getWidth() / 2, y = this._getHeight() / 2;
        return { x: x, y: y };
    };
    NgxMGaugeComponent.prototype._init = function () {
        this._context2d = this._canvas.nativeElement.getContext('2d');
        this._initialized = true;
        this._refreshSize();
        this._configStyles();
        this._create();
    };
    NgxMGaugeComponent.prototype._destroy = function () {
        this._clear();
        this._context2d = null;
    };
    NgxMGaugeComponent.prototype._configStyles = function () {
        if (this._context2d) {
            this._context2d.canvas.width = this.options.size;
            this._context2d.canvas.height = this.options.size;
            this._context2d.lineCap = this.options.cap.toString();
            this._context2d.lineWidth = this.options.thick;
        }
    };
    NgxMGaugeComponent.prototype._getForegroundColorByRange = function (value) {
        if (this.thresholds) {
            var match = Object.keys(this.thresholds)
                .filter(function (item) { return isNumber(item) && Number(item) <= Number(value); })
                .sort().reverse()[0];
            return match !== undefined
                ? this.thresholds[match].color || this.options.foregroundColor
                : this.options.foregroundColor;
        }
        else {
            return this.options.foregroundColor;
        }
    };
    NgxMGaugeComponent.prototype._create = function () {
        var self = this, type = this.options.type, bounds = this._getBounds(type), duration = this.duration, min = this.min, max = this.max, value = clamp(this.value, this.min, this.max), head = bounds.head, unit = (bounds.tail - bounds.head) / (max !== min ? (max - min) : 1), displacement = unit * (value - min), tail = bounds.tail, color = this._getForegroundColorByRange(value), requestID, starttime;
        function animate(timestamp) {
            timestamp = timestamp || new Date().getTime();
            var runtime = timestamp - starttime;
            var progress = runtime / duration;
            progress = Math.min(progress, 1);
            self._drawShell(head, head + displacement * progress, tail, color);
            if (runtime < duration) {
                requestID = window.requestAnimationFrame(function (ts) { return animate(ts); });
            }
            else {
                window.cancelAnimationFrame(requestID);
            }
        }
        window.requestAnimationFrame(function (timestamp) {
            starttime = timestamp || new Date().getTime();
            animate(timestamp);
        });
    };
    NgxMGaugeComponent.prototype._refresh = function () {
        this._clear();
        this._create();
    };
    NgxMGaugeComponent.decorators = [
        { type: _angular_core.Component, args: [{
                    selector: 'ngx-mgauge',
                    template: "\n    <div #reading class=\"block\"\n         [style.fontSize]=\"options.size * 0.16 + 'px'\"\n         [style.lineHeight]=\"options.size + 'px'\"\n         [style.width.px]=\"options.size\"\n         [style.height.px]=\"options.size\">\n        <u class=\"affix\">\n            <span>{{prepend}}</span>\n        </u>\n        <span>{{value | number}}</span>\n        <u *ngIf=\"append\" class=\"affix\">\n            <span>{{append}}</span>\n        </u>\n    </div>\n    <div class=\"label\"\n         [style.fontSize]=\"options.size / 10 + 'px'\"\n         [style.lineHeight]=\"(5 * options.size / 14) + options.size + 'px'\"\n         [style.width.px]=\"options.size\"\n         [style.height.px]=\"options.size\">\n        <span>{{label}}</span>\n    </div>\n    <div *ngIf=\"options.showExtremum === true && options.type !== 'full'\"\n         class=\"min\"\n         [style.fontSize]=\"options.size / 9 + 'px'\"\n         [style.lineHeight]=\"(6 * options.size / (options.type === 'semi' ? 20 : 7.5)) + options.size + 'px'\"\n         [style.paddingLeft]=\"(options.type === 'semi' ? 5 : 15) + 'px'\"\n         [style.width.px]=\"options.size\"\n         [style.height.px]=\"options.size\">\n        <span>{{min}}</span>\n    </div>\n    <div *ngIf=\"options.showExtremum === true && options.type !== 'full'\"\n         class=\"max\"\n         [style.fontSize]=\"options.size / 9 + 'px'\"\n         [style.lineHeight]=\"(6 * options.size / (options.type === 'semi' ? 20 : 7.5)) + options.size + 'px'\"\n         [style.paddingRight]=\"(options.type === 'semi' ? 5 : 15) + 'px'\"\n         [style.width.px]=\"options.size\"\n         [style.height.px]=\"options.size\">\n        <span>{{max}}</span>\n    </div>\n    <canvas #canvas\n            [width]=\"options.size\"\n            [height]=\"options.size\"></canvas>\n  ",
                    styles: ["\n    .mgauge{display:inline-block;text-align:center;position:relative}.block,.label{position:absolute;display:inline-block;font-weight:400;white-space:nowrap;overflow:hidden;text-overflow:ellipsis;text-align:center}.label{font-family:inherit}.min{text-align:left;padding-left:15px}.max,.min{font-family:inherit;position:absolute;display:inline-block;white-space:nowrap;overflow:hidden;text-overflow:ellipsis;font-weight:400}.max{text-align:right;padding-right:15px}.affix{text-decoration:none;font-size:.6em;opacity:.8;font-weight:200}.affix:first-child{padding-left:0}.affix:last-child{padding-right:0}\n  "],
                    encapsulation: _angular_core.ViewEncapsulation.Emulated
                },] },
    ];
    /** @nocollapse */
    NgxMGaugeComponent.ctorParameters = [
        { type: _angular_core.ElementRef, },
        { type: _angular_core.Renderer, },
    ];
    NgxMGaugeComponent.propDecorators = {
        'role': [{ type: _angular_core.HostBinding, args: ['attr.role',] },],
        'gaugeMeterClass': [{ type: _angular_core.HostBinding, args: ['class.mgauge',] },],
        'ariaValueminAttr': [{ type: _angular_core.HostBinding, args: ['attr.aria-valuemin',] },],
        'ariaValuemaxAttr': [{ type: _angular_core.HostBinding, args: ['attr.aria-valuemax',] },],
        'ariaValueNowAttr': [{ type: _angular_core.HostBinding, args: ['attr.aria-valuenow',] },],
        '_canvas': [{ type: _angular_core.ViewChild, args: ['canvas',] },],
        'min': [{ type: _angular_core.Input },],
        'max': [{ type: _angular_core.Input },],
        'label': [{ type: _angular_core.Input },],
        'append': [{ type: _angular_core.Input },],
        'prepend': [{ type: _angular_core.Input },],
        'thresholds': [{ type: _angular_core.Input },],
        'options': [{ type: _angular_core.Input },],
        'value': [{ type: _angular_core.Input },],
        'duration': [{ type: _angular_core.Input },],
        'showExtremum': [{ type: _angular_core.Input },],
    };
    return NgxMGaugeComponent;
}());

var NgxMGaugeModule = (function () {
    function NgxMGaugeModule() {
    }
    NgxMGaugeModule.forRoot = function (config) {
        return {
            ngModule: NgxMGaugeModule,
            providers: [{ provide: NgxMGaugeOptions, useValue: config }]
        };
    };
    NgxMGaugeModule.decorators = [
        { type: _angular_core.NgModule, args: [{
                    imports: [
                        _angular_common.CommonModule
                    ],
                    exports: [NgxMGaugeComponent],
                    declarations: [NgxMGaugeComponent]
                },] },
    ];
    /** @nocollapse */
    NgxMGaugeModule.ctorParameters = [];
    return NgxMGaugeModule;
}());

exports.DEFAULTS = DEFAULTS;
exports.NgxMGaugeOptions = NgxMGaugeOptions;
exports.NgxMGaugeModule = NgxMGaugeModule;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-mgauge.umd.js.map
